/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "hardwareannotationplugin.h"
#include "utils.h"
#include "swimconnection.h"
#include "lino.h"
#include "wf.h"

#include <Nepomuk/Thing>
#include <Nepomuk/Variant>
#include <Nepomuk/ResourceManager>
#include <nepomuk/simpleannotation.h>

#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/QueryResultIterator>
#include <Soprano/Util/AsyncResult>

#include <KDebug>


namespace {
    class HardwareAnnotation : public Nepomuk::SimpleAnnotation
    {
    public:
        HardwareAnnotation( const Nepomuk::Resource& res, const QUrl& app, const QString& title );

    protected:
        void doCreate();

    private:
        Nepomuk::Thing m_component;
        QString m_title;
    };

    HardwareAnnotation::HardwareAnnotation( const Nepomuk::Resource& res, const QUrl& component, const QString& title )
        : SimpleAnnotation( res ),
          m_component( component ),
          m_title( title ) {
        setProperty( Lino::Vocabulary::LINO::hardware() );
        setValue( Nepomuk::Variant( m_component ) );
        setLabel( title );
        setComment( i18n( "Is related to hardware '%1'", title ) );
    }

    void HardwareAnnotation::doCreate()
    {
        m_component.setLabel( m_title );
        m_component.addType( Lino::Vocabulary::LINO::Hardware() );
        SimpleAnnotation::doCreate();
    }
}


LinoHardwareAnnotationPlugin::LinoHardwareAnnotationPlugin( QObject* parent, const QVariantList& )
    : AnnotationPlugin( parent ),
      m_lastResult( 0 )
{
    kDebug();
}


LinoHardwareAnnotationPlugin::~LinoHardwareAnnotationPlugin()
{
}


void LinoHardwareAnnotationPlugin::doGetPossibleAnnotations( const Nepomuk::Resource& resource, const QString& filter )
{
    kDebug() << resource << filter;

    m_resource = resource;
    m_filter = filter;

    if ( m_lastResult )
        m_lastResult->disconnect( this );

    if ( filter.length() ) {
        // do a remote query for hardware
        QString query = QString( "select ?uri ?compLabel where { "
                                 "?uri a <%1> . "
                                 "{ ?uri <%2> ?compLabel . "
                                 "?compLabel bif:contains '%3' . } "
                                 "UNION "
                                 "{ ?uri <%4> ?manuf . "
                                 "?uri <%2> ?compLabel . "
                                 "?manuf <%2> ?manufLabel . "
                                 "?manufLabel bif:contains '%3' . } "
                                 "} limit 20" )
                        .arg( Lino::Vocabulary::LINO::Hardware().toString() )
                        .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                        .arg( Lino::buildVirtuosoFullTextQuery( filter ) )
                        .arg( Lino::Vocabulary::LINO::hardwareManufacturer().toString() );
        m_lastResult = SwimConnection::instance()->executeQueryAsync( query );
        connect( m_lastResult, SIGNAL( resultReady( Soprano::Util::AsyncResult* ) ),
                 this, SLOT( slotHardwareQueryResult( Soprano::Util::AsyncResult* ) ) );
    }
    else
        m_lastResult = 0;
}


void LinoHardwareAnnotationPlugin::slotHardwareQueryResult( Soprano::Util::AsyncResult* result )
{
    kDebug();

    m_lastResult = 0;

    Soprano::QueryResultIterator it = result->queryResultIterator();
    while ( it.next() ) {
        QUrl comp = it["uri"].uri();
        QString label = it["compLabel"].toString();
        HardwareAnnotation* anno = new HardwareAnnotation( m_resource, comp, label );
        anno->setRelevance( 1.0 - Lino::calculateDistance( m_filter, label ) );
        addNewAnnotation( anno );
    }
}


NEPOMUK_EXPORT_ANNOTATION_PLUGIN( LinoHardwareAnnotationPlugin, "nepomuk_linohardwareannotationplugin" )

#include "hardwareannotationplugin.moc"
