/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "softwareannotationplugin.h"
#include "utils.h"
#include "swimconnection.h"
#include "lino.h"
#include "wf.h"

#include <Nepomuk/Thing>
#include <Nepomuk/Variant>
#include <Nepomuk/ResourceManager>
#include <nepomuk/simpleannotation.h>

#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/QueryResultIterator>
#include <Soprano/Util/AsyncResult>

#include <KDebug>


namespace {
    class SoftwareAnnotation : public Nepomuk::SimpleAnnotation
    {
    public:
        SoftwareAnnotation( const Nepomuk::Resource& res, const QUrl& app, const QString& title );

    protected:
        void doCreate();

    private:
        Nepomuk::Thing m_app;
        QString m_title;
    };

    SoftwareAnnotation::SoftwareAnnotation( const Nepomuk::Resource& res, const QUrl& app, const QString& title )
        : SimpleAnnotation( res ),
          m_app( app ),
          m_title( title ) {
        setProperty( Lino::Vocabulary::LINO::software() );
        setValue( Nepomuk::Variant( m_app ) );
        setLabel( title );
        setComment( i18n( "Is related to software '%1'", title ) );
    }

    void SoftwareAnnotation::doCreate()
    {
        m_app.setLabel( m_title );
        m_app.addType( Lino::Vocabulary::LINO::Software() );
        SimpleAnnotation::doCreate();
    }
}


LinoSoftwareAnnotationPlugin::LinoSoftwareAnnotationPlugin( QObject* parent, const QVariantList& )
    : AnnotationPlugin( parent ),
      m_lastResult( 0 )
{
    kDebug();
}


LinoSoftwareAnnotationPlugin::~LinoSoftwareAnnotationPlugin()
{
}


// either move this to another plugin or rename the plugin
void LinoSoftwareAnnotationPlugin::getPossibleStateAnnotations()
{
    // propose to close the bug or forum post
    QUrl type = Lino::determineResourceTypeFromUrl( m_resource.resourceUri() );
    if ( type == Lino::Vocabulary::LINO::Bug() ||
         type == Lino::Vocabulary::LINO::ForumThread() ||
         m_resource.types().contains( Lino::Vocabulary::LINO::Bug() ) ||
         m_resource.types().contains( Lino::Vocabulary::LINO::ForumThread() ) ) {
        Soprano::QueryResultIterator it
            = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString( "select ?state ?label where { "
                                                                                        "?state a <%1> . "
                                                                                        "?state <%2> ?label . "
                                                                                        "filter(regex(str(?label), '%3', 'i')) . "
                                                                                        "} limit 5" )
                                                                               .arg( Lino::Vocabulary::WF::State().toString() )
                                                                               .arg( Soprano::Vocabulary::RDFS::label().toString() )
                                                                               .arg( m_filter ),
                                                                               Soprano::Query::QueryLanguageSparql );
        while ( it.next() ) {
            QUrl state = it["state"].uri();
            QString label = it["label"].toString();
            Nepomuk::SimpleAnnotation* anno = new Nepomuk::SimpleAnnotation( m_resource,
                                                                             Lino::Vocabulary::WF::state(),
                                                                             Nepomuk::Resource( state ) );
            anno->setLabel( "resolved" );
            anno->setComment( i18n( "Mark as '%1'.", label ) );
            anno->setRelevance( 1.0 - Lino::calculateDistance( m_filter, label ) );
            addNewAnnotation( anno );
        }
    }
}


void LinoSoftwareAnnotationPlugin::getPossibleDistAnnotations()
{
    // do a local query on the lino ontology which contains the distributions
    QString query = QString( "select ?uri ?label where { "
                             "?uri a ?type . "
                             "?type <%1> <%2> . "
                             "?uri <%3> ?label . "
                             "filter(regex(str(?label), '%4','i')) . "
                             "} limit 5" )
                    .arg( Soprano::Vocabulary::RDFS::subClassOf().toString() )
                    .arg( Lino::Vocabulary::LINO::Linux().toString() )
                    .arg( Soprano::Vocabulary::RDFS::label().toString() )
                    .arg( m_filter );
    kDebug() << query;
    Soprano::QueryResultIterator it
        = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( query,
                                                                           Soprano::Query::QueryLanguageSparql );
    while ( it.next() ) {
        QUrl dist = it["uri"].uri();
        QString label = it["label"].toString();
        Nepomuk::SimpleAnnotation* anno = new Nepomuk::SimpleAnnotation( m_resource,
                                                                         Lino::Vocabulary::LINO::isRelatedToOS(),
                                                                         Nepomuk::Variant( Nepomuk::Resource( dist ) ) );
        anno->setLabel( label );
        anno->setComment( i18n( "Is related to linux distribution '%1'", label ) );
        anno->setRelevance( 1.0 - Lino::calculateDistance( m_filter, label ) );
        addNewAnnotation( anno );
    }
}


void LinoSoftwareAnnotationPlugin::doGetPossibleAnnotations( const Nepomuk::Resource& resource, const QString& filter )
{
    kDebug() << resource << filter;

    m_resource = resource;
    m_filter = filter;

    if ( m_lastResult )
        m_lastResult->disconnect( this );

    getPossibleStateAnnotations();
    getPossibleDistAnnotations();

    if ( filter.length() ) {
        // do a remote query for software
        m_lastResult = SwimConnection::instance()->executeQueryAsync( QString( "select ?uri ?label where { "
                                                                               "?uri a <%1> . "
                                                                               "?uri <%2> ?label . "
                                                                               "?label bif:contains '%3' . "
                                                                               "} limit 5" )
                                                                      .arg( Lino::Vocabulary::LINO::Software().toString() )
                                                                      .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                                                                      .arg( Lino::buildVirtuosoFullTextQuery( filter ) ) );
        connect( m_lastResult, SIGNAL( resultReady( Soprano::Util::AsyncResult* ) ),
                 this, SLOT( slotSoftwareQueryResult( Soprano::Util::AsyncResult* ) ) );
    }
    else
        m_lastResult = 0;
}


void LinoSoftwareAnnotationPlugin::slotSoftwareQueryResult( Soprano::Util::AsyncResult* result )
{
    kDebug();

    m_lastResult = 0;

    Soprano::QueryResultIterator it = result->queryResultIterator();
    while ( it.next() ) {
        QUrl app = it["uri"].uri();
        QString label = it["label"].toString();
        SoftwareAnnotation* anno = new SoftwareAnnotation( m_resource, app, label );
        anno->setRelevance( 1.0 - Lino::calculateDistance( m_filter, label ) );
        addNewAnnotation( anno );
    }
}


NEPOMUK_EXPORT_ANNOTATION_PLUGIN( LinoSoftwareAnnotationPlugin, "nepomuk_linosoftwareannotationplugin" )

#include "softwareannotationplugin.moc"
