/* This file is part of the Nepomuk Project
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "bugloader.h"
#include "bom.h"
#include "lino.h"
#include "bug.h"
#include "settings.h"
#include "swimconnection.h"

#include <QtCore/QTimer>

#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Util/AsyncResult>

#include <KDebug>


static const int s_batchSize = 50;

class Lino::BugLoader::Private
{
public:
    int bugCount;
    QString query;
};


Lino::BugLoader::BugLoader( QObject* parent )
    : KJob( parent ),
      d( new Private() )
{
}


Lino::BugLoader::~BugLoader()
{
    delete d;
}


void Lino::BugLoader::runQuery( const QString& query )
{
    d->bugCount = 0;
    d->query = query;

    QTimer::singleShot( 0, this, SLOT( loadNextBatch() ) );
}


void Lino::BugLoader::start()
{
    query( QString(), QString() );
}


void Lino::BugLoader::query( const QString& app, const QString& query )
{
    QString s = QString( "select * where { "
                         "?bug a <%1> . "
                         "?bug <%2> ?title . "
                         "?bug <%3> ?link . "
                         "OPTIONAL { ?bug <http://www.w3.org/2005/01/wf/flow#state> ?state . } " )
                .arg( Lino::Vocabulary::BOM::Issue().toString() )
                .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                .arg( Lino::Vocabulary::LINO::bugzilla().toString() );

    if ( !app.isEmpty() ) {
        s += QString( "?bug <%1> ?app . "
                      "?app <%2> ?appTitle . "
                      "FILTER(bif:contains(?appTitle, \"%3\")) . " )
             .arg( Lino::Vocabulary::BOM::isIssueOf().toString() )
             .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
             .arg( app );
    }

    if ( !query.isEmpty() ) {
        s += QString( "FILTER(bif:contains(?title, \"%1\")) ." ).arg( query );
    }

    s += " }";

    runQuery( s );
}


void Lino::BugLoader::loadNextBatch()
{
    connect( SwimConnection::instance()->executeQueryAsync( QString( "%1 offset %2 limit %3" )
                                                            .arg( d->query )
                                                            .arg( d->bugCount )
                                                            .arg( s_batchSize ), Soprano::Query::QueryLanguageSparql ),
             SIGNAL(resultReady(Soprano::Util::AsyncResult*)),
             this, SLOT(slotNextBatchReady(Soprano::Util::AsyncResult*)) );
}


void Lino::BugLoader::slotNextBatchReady( Soprano::Util::AsyncResult* result )
{
    if ( result->errorCode() == Soprano::Error::ErrorNone ) {
        QList<Lino::Bug> bugs;
        Soprano::QueryResultIterator it = result->queryResultIterator();
        while ( it.next() ) {
            Lino::Bug bug;
            bug.setTitle( it["title"].toString() );
            bug.setBugReportUrl( it["link"].toString() );
            bug.setResourceUri( it["bug"].uri() );
            bug.setState( it["state"].uri() );
            bugs << bug;
            kDebug() << "Loaded bug" << bug.resourceUri();
        }

        if ( bugs.isEmpty() ) {
            emitResult();
        }
        else {
            kDebug() << "Loaded" << bugs.count() << "bugs";
            emit bugsLoaded( bugs );
            d->bugCount += s_batchSize;
            QTimer::singleShot( 0, this, SLOT( loadNextBatch() ) );
        }
    }
    else {
        setErrorText( result->lastError().message() );
        emitResult();
    }
}

#include "bugloader.moc"
