/* This file is part of the Nepomuk Project
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "linobrowser.h"
#include "linobrowserview.h"
#include "settings.h"
#include "urlhistory.h"
#include "querywidget.h"
#include "annotationuploader.h"
#include "bug.h"
#include "swimconnection.h"

#include <QtGui/QDropEvent>
#include <QtGui/QPainter>

#include <kconfigdialog.h>
#include <kstatusbar.h>

#include <kaction.h>
#include <kactioncollection.h>
#include <kstandardaction.h>

#include <KLocale>
#include <KIcon>
#include <KMessageBox>


LinoBrowser::LinoBrowser()
    : KXmlGuiWindow(),
      m_view(new LinoBrowserView(this)),
      m_queryWidget(0)
{
    m_urlHistory = new UrlHistory( this );
    connect( m_urlHistory, SIGNAL( urlChanged( QUrl ) ),
             m_view, SLOT( setBug( QUrl ) ) );
    connect( m_view, SIGNAL( bugSelected( QUrl ) ),
             m_urlHistory, SLOT( setUrl( QUrl ) ) );

    // accept dnd
    setAcceptDrops(true);

    // tell the KXmlGuiWindow that this is indeed the main widget
    setCentralWidget(m_view);

    setupQueryWidget();

    // then, setup our actions
    setupActions();

    // add a status bar
    statusBar()->show();

    // a call to KXmlGuiWindow::setupGUI() populates the GUI
    // with actions, using KXMLGUI.
    // It also applies the saved mainwindow settings, if any, and ask the
    // mainwindow to automatically save settings if changed: window size,
    // toolbar position, icon size, etc.
    setupGUI();
}


LinoBrowser::~LinoBrowser()
{
}


void LinoBrowser::setupActions()
{
    KStandardAction::quit(qApp, SLOT(closeAllWindows()), actionCollection());
    KStandardAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
    KAction* reloadAction = KStandardAction::redisplay( this, SLOT(loadAllBugs()), actionCollection() );
    reloadAction->setText( i18n( "Reload all bugs" ) );
    actionCollection()->addAction( "go_back", m_urlHistory->backwardAction() );
    actionCollection()->addAction( "go_forward", m_urlHistory->forwardAction() );
    QAction* queryAction = m_queryWidget->toggleViewAction();
    queryAction->setText( i18n( "Find Bugs" ) );
    queryAction->setIconText( i18n( "Find Bugs" ) );
    queryAction->setIcon( KIcon( "edit-find" ) );
    actionCollection()->addAction( "edit_find", queryAction );
    KAction* uploadAnnotationAction = new KAction( KIcon("arrow-up"), i18n("Upload Bug Annotations"), actionCollection() );
    actionCollection()->addAction("upload_annotations", uploadAnnotationAction );
    connect(uploadAnnotationAction, SIGNAL(triggered()), this, SLOT(slotUploadAnnotations()) );
}


void LinoBrowser::optionsPreferences()
{
    // The preference dialog is derived from prefs_base.ui
    //
    // compare the names of the widgets in the .ui file
    // to the names of the variables in the .kcfg file
    // avoid to have 2 dialogs shown
    if ( KConfigDialog::showDialog( "settings" ) )  {
        return;
    }
    KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
    QWidget *generalSettingsDlg = new QWidget;
    ui_prefs_base.setupUi(generalSettingsDlg);
    dialog->addPage(generalSettingsDlg, i18n("General"), "package_setting");
    connect(dialog, SIGNAL(settingsChanged(QString)), SwimConnection::instance(), SLOT(updateConnectionSettings()));
    dialog->show();
}


void LinoBrowser::setupQueryWidget()
{
    if ( !m_queryWidget ) {
        m_queryWidget = new QueryWidget( this );
        connect( m_queryWidget, SIGNAL( queryExecuted( QString, QString ) ),
                 m_view, SLOT( queryBugs( QString, QString ) ) );
        addDockWidget( Qt::TopDockWidgetArea, m_queryWidget );
        m_queryWidget->hide();
    }
}


void LinoBrowser::loadAllBugs()
{
    m_queryWidget->hide();
    m_view->reload();
}


void LinoBrowser::slotUploadAnnotations()
{
    Lino::Bug bug = m_view->selectedBug();
    if( bug.isValid() ) {
        if( KMessageBox::questionYesNo( this, i18n("Do you want to upload the local annotations (tags and others) to the SWIM server?" ) ) == KMessageBox::Yes ) {
            Lino::AnnotationUploader* job = Lino::AnnotationUploader::uploadAnnotations( bug.localResource() );
            job->start();
        }
    }
    else {
        KMessageBox::sorry( this, i18n("Please select a bug") );
    }
}

#include "linobrowser.moc"
