/* This file is part of the Nepomuk Project
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "linobrowserview.h"
#include "linobrowser.h"
#include "settings.h"
#include "bugmodel.h"
#include "bug.h"
#include "bugloader.h"
#include "lino.h"
#include "utils.h"
#include "bugfiltermodel.h"
#include "recommendedbugmodel.h"

#include <KLocale>
#include <KIcon>
#include <KFilterProxySearchLine>
#include <KMessageBox>
#include <KStatusBar>

#include <QtGui/QLabel>
#include <QtGui/QSortFilterProxyModel>

Q_DECLARE_METATYPE( QList<QUrl> )

static const int s_statusbarIdBugCount = 1;
static const int s_statusbarIdProgress = 2;

class StringAnimator {
public:
    StringAnimator( const QString& s )
        : m_string( s ),
          m_dc( -1 ) {
    }

    QString animatedString() {
        ++m_dc;
        if ( m_dc > 3 ) {
            m_dc = -2;
        }
        return m_string.leftJustified( m_string.length() + qAbs( m_dc ), '.' );
    }

    void reset() {
        m_dc = -1;
    }

private:
    QString m_string;
    int m_dc;
};


LinoBrowserView::LinoBrowserView( LinoBrowser* parent )
    : QWidget( parent ),
      m_mainWindow( parent ),
      m_runningBugLoader( 0 )
{
    setupUi(this);
    setAutoFillBackground(false);

    m_stringAnimator = new StringAnimator( i18n( "Loading bugs" ) );
    connect( &m_animationTimer, SIGNAL( timeout() ),
             this, SLOT( updateStatusBar() ) );

    m_bugModel = new Lino::BugModel( m_viewBugs );
    m_sortModel = new Lino::BugFilterModel( m_viewBugs );
    m_sortModel->setSourceModel( m_bugModel );
    m_viewBugs->setModel( m_sortModel );
    m_viewBugs->setSortingEnabled( true );
    m_lineFilterBugs->setProxy( m_sortModel );

    m_recommendedBugsModel = new Lino::RecommendedBugModel( m_viewRelatedBugs );
    m_viewRelatedBugs->setModel( m_recommendedBugsModel );

    connect( m_bugModel, SIGNAL(rowsInserted(QModelIndex, int, int) ),
             this, SLOT( slotBugModelChanged() ) );
    connect( m_bugModel, SIGNAL(modelReset()),
             this, SLOT( slotBugModelChanged() ) );
    connect( m_viewBugs->selectionModel(), SIGNAL( currentChanged( QModelIndex, QModelIndex ) ),
             this, SLOT( slotCurrentBugChanged( QModelIndex, QModelIndex ) ) );
    connect( m_viewRelatedBugs, SIGNAL( doubleClicked( QModelIndex ) ),
             this, SLOT( slotRelatedBugExecuted( QModelIndex ) ) );
    connect( m_comboState, SIGNAL( activated( int ) ),
             this, SLOT( slotStateFilterChanged( int ) ) );
    connect( m_taggingWidget, SIGNAL(tagsChanged(QList<Nepomuk::Tag>)),
             this, SLOT(slotTagsChanged()) );

    // setup states
    m_comboState->addItem( i18nc( "@label:listbox Filter for bugs", "All States" ),
                           ( QVariant::fromValue( QList<QUrl>() ) ) );
    m_comboState->addItem( i18nc( "@label:listbox Filter for bugs", "All Open States" ),
                           ( QVariant::fromValue( QList<QUrl>()
                             << Lino::Vocabulary::LINO::Unconfirmed()
                             << Lino::Vocabulary::LINO::New()
                             << Lino::Vocabulary::LINO::Assigned()
                             << Lino::Vocabulary::LINO::Reopened() ) ) );
    m_comboState->addItem( i18nc( "@label:listbox Filter for bugs", "All Closed States" ),
                           ( QVariant::fromValue( QList<QUrl>()
                             << Lino::Vocabulary::LINO::Resolved()
                             << Lino::Vocabulary::LINO::Verified()
                             << Lino::Vocabulary::LINO::Closed() ) ) );
    m_comboState->addItem( Lino::bugStateString( Lino::Vocabulary::LINO::Unconfirmed() ),
                           QVariant::fromValue( QList<QUrl>() << Lino::Vocabulary::LINO::Unconfirmed() ) );
    m_comboState->addItem( Lino::bugStateString( Lino::Vocabulary::LINO::New() ),
                           QVariant::fromValue( QList<QUrl>() << Lino::Vocabulary::LINO::New() ) );
    m_comboState->addItem( Lino::bugStateString( Lino::Vocabulary::LINO::Assigned() ),
                           QVariant::fromValue( QList<QUrl>() << Lino::Vocabulary::LINO::Assigned() ) );
    m_comboState->addItem( Lino::bugStateString( Lino::Vocabulary::LINO::Reopened() ),
                           QVariant::fromValue( QList<QUrl>() << Lino::Vocabulary::LINO::Reopened() ) );
    m_comboState->addItem( Lino::bugStateString( Lino::Vocabulary::LINO::Resolved() ),
                           QVariant::fromValue( QList<QUrl>() << Lino::Vocabulary::LINO::Resolved() ) );
    m_comboState->addItem( Lino::bugStateString( Lino::Vocabulary::LINO::Closed() ),
                           QVariant::fromValue( QList<QUrl>() << Lino::Vocabulary::LINO::Closed() ) );

    updateStatusBar();
    slotCurrentBugChanged( QModelIndex(), QModelIndex() );
}


LinoBrowserView::~LinoBrowserView()
{
    delete m_stringAnimator;
}


Lino::Bug LinoBrowserView::selectedBug() const
{
    return m_bugModel->bugForIndex( m_sortModel->mapToSource( m_viewBugs->selectionModel()->currentIndex() ) );
}


void LinoBrowserView::setBug( const Lino::Bug& bug )
{
    m_viewBugs->selectionModel()->blockSignals( true );

    // it might happend that slotRelatedBugExecuted sets a bug which is not loaded yet
    // FIXME: load the one bug in that case
    QModelIndex index = m_sortModel->mapFromSource( m_bugModel->indexForBug( bug ) );
    if ( index.isValid() ) {
        m_viewBugs->selectionModel()
            ->setCurrentIndex( index, QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Current );
    }
    else {
        m_viewBugs->clearSelection();
    }
    m_webViewBug->setUrl( bug.bugReportUrl() );
    m_taggingWidget->setResource( bug.localResource().pimoThing() );
    m_recommendedBugsModel->setParentBug( bug );
    m_taggingWidget->setEnabled( bug.isValid() );

    m_viewBugs->selectionModel()->blockSignals( false );
}


void LinoBrowserView::setBug( const QUrl& url )
{
    QModelIndex index = m_bugModel->indexForBug( url );
    setBug( m_bugModel->bugForIndex( index ) );
}


void LinoBrowserView::slotCurrentBugChanged( const QModelIndex& current, const QModelIndex& )
{
    Lino::Bug bug = m_bugModel->bugForIndex( m_sortModel->mapToSource( current ) );
    kDebug() << "current bug changed:" << bug.title() << bug.bugReportUrl();
    setBug( bug );
    emit bugSelected( bug.resourceUri() );
}


void LinoBrowserView::slotRelatedBugExecuted( const QModelIndex& index )
{
    setBug( m_recommendedBugsModel->bugForIndex( index ) );
}


void LinoBrowserView::reload()
{
    m_animationTimer.start( 1000 );

    if ( m_runningBugLoader ) {
        m_runningBugLoader->disconnect( this );
    }

    m_bugModel->clear();

    Lino::BugLoader* loader = new Lino::BugLoader( this );
    connect( loader, SIGNAL( bugsLoaded( QList<Lino::Bug> ) ),
             m_bugModel, SLOT( addBugs( QList<Lino::Bug> ) ) );
    connect( loader, SIGNAL( result( KJob* ) ),
             this, SLOT( slotBugLoaderFinished( KJob* ) ) );
    loader->start(); // job will delete itself
    m_runningBugLoader = loader;
}


void LinoBrowserView::queryBugs( const QString& app, const QString& query )
{
    m_animationTimer.start( 1000 );

    if ( m_runningBugLoader ) {
        m_runningBugLoader->disconnect( this );
    }

    m_bugModel->clear();

    Lino::BugLoader* loader = new Lino::BugLoader( this );
    connect( loader, SIGNAL( bugsLoaded( QList<Lino::Bug> ) ),
             m_bugModel, SLOT( addBugs( QList<Lino::Bug> ) ) );
    connect( loader, SIGNAL( result( KJob* ) ),
             this, SLOT( slotBugLoaderFinished( KJob* ) ) );
    loader->query( app, query ); // job will delete itself
    m_runningBugLoader = loader;
}


void LinoBrowserView::slotBugLoaderFinished( KJob* job )
{
    m_animationTimer.stop();
    updateStatusBar();

    Lino::BugLoader* loader = static_cast<Lino::BugLoader*>( job );
    if ( loader->error() ) {
        KMessageBox::error( this,
                            i18n( "An error occured during bug loading: %1", loader->errorString() ) );
    }
    else {
        KMessageBox::information( this,
                                  i18np( "1 bug loaded", "%1 bugs loaded", m_bugModel->rowCount( QModelIndex() ) ) );
    }
}


void LinoBrowserView::slotStateFilterChanged( int i )
{
    m_sortModel->setStateFilter( m_comboState->itemData( i ).value<QList<QUrl> >() );
}


void LinoBrowserView::slotBugModelChanged()
{
    updateStatusBar();
    m_viewBugs->scrollTo( m_viewBugs->selectionModel()->currentIndex(), QAbstractItemView::EnsureVisible );
}


void LinoBrowserView::updateStatusBar()
{
    QString bugCountStr = i18np( "One bug loaded", "%1 bugs loaded", m_bugModel->rowCount(QModelIndex()) );
    if ( !m_mainWindow->statusBar()->hasItem( s_statusbarIdBugCount ) ) {
        m_mainWindow->statusBar()->insertPermanentItem( bugCountStr, s_statusbarIdBugCount );
    }
    else {
        m_mainWindow->statusBar()->changeItem( bugCountStr, s_statusbarIdBugCount );
    }
    if ( m_animationTimer.isActive() ) {
        if ( !m_mainWindow->statusBar()->hasItem( s_statusbarIdProgress ) ) {
            m_stringAnimator->reset();
            m_mainWindow->statusBar()->insertPermanentFixedItem( m_stringAnimator->animatedString() + "...", s_statusbarIdProgress );
            m_mainWindow->statusBar()->setItemAlignment( s_statusbarIdProgress, Qt::AlignLeft );
            m_stringAnimator->reset();
        }
        m_mainWindow->statusBar()->changeItem( m_stringAnimator->animatedString(), s_statusbarIdProgress );
    }
    else if ( m_mainWindow->statusBar()->hasItem( s_statusbarIdProgress ) ) {
        m_mainWindow->statusBar()->removeItem( s_statusbarIdProgress );
    }
}


void LinoBrowserView::slotTagsChanged()
{
    // set the label of the thing only after changing the tags.
    // otherwise each selected bug would be created in the local nepomuk store
    Lino::Bug bug = selectedBug();
    bug.localResource().pimoThing().setLabel( bug.title() );
    bug.localResource().pimoThing().addType( Lino::Vocabulary::LINO::Bug() );
}

#include "linobrowserview.moc"
