/* This file is part of the Nepomuk Project
   Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "recommendedbugmodel.h"
#include "settings.h"
#include "bom.h"
#include "lino.h"

#include <KLocale>

#include <Soprano/Client/SparqlModel>
#include <Soprano/Util/AsyncResult>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/QueryResultIterator>


namespace {
    QStringList extractKeywordsTheStupidWay( const QString& s )
    {
        QStringList keywords = s.split( QRegExp("[\\s\\(\\)\\.\\,\\;\\:]"), QString::SkipEmptyParts );
        QStringList::iterator it = keywords.begin();
        while ( it != keywords.end() ) {
            if ( it->length() < 4 )
                it = keywords.erase( it );
            else ++it;
        }
        return keywords;
    }
}


Lino::RecommendedBugModel::RecommendedBugModel( QObject* parent )
    : BugModel( parent ),
      m_currentResult( 0 )
{
    QUrl dbUrl = Settings::database();
    m_model = new Soprano::Client::SparqlModel();
    m_model->setHost( dbUrl.host(), dbUrl.port() );
    m_model->setUser( dbUrl.userName(), dbUrl.password() );
}


Lino::RecommendedBugModel::~RecommendedBugModel()
{
    delete m_model;
}


void Lino::RecommendedBugModel::setParentBug( const Bug& bug )
{
    m_parentBug = bug;
    if ( m_currentResult ) {
        disconnect( m_currentResult );
    }
    m_requestFinished = false;
    BugModel::clear();

    QStringList keywords = extractKeywordsTheStupidWay( bug.title() );
    if ( !keywords.isEmpty() ) {
        QString query = QString( "select ?bug ?title where { "
                                 "?bug a <%1> . "
                                 "?bug <%2> ?title . "
                                 "?title bif:contains '%3' . } LIMIT 10" )
                        .arg( Vocabulary::BOM::Issue().toString() )
                        .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                        .arg( keywords.join( " OR " ) );
        m_currentResult = m_model->executeQueryAsync( query, Soprano::Query::QueryLanguageSparql );
        connect( m_currentResult, SIGNAL(resultReady( Soprano::Util::AsyncResult* ) ),
                 this, SLOT( slotBugQueryReady( Soprano::Util::AsyncResult* ) ) );
    }
    else {
        m_requestFinished = true;
        reset();
    }
}


void Lino::RecommendedBugModel::slotBugQueryReady( Soprano::Util::AsyncResult* result )
{
    m_currentResult = 0;
    m_requestFinished = true;
    Soprano::QueryResultIterator it = result->queryResultIterator();
    QList<Lino::Bug> bugs;
    while ( it.next() ) {
        Lino::Bug bug;
        bug.setTitle( it["title"].toString() );
        bug.setResourceUri( it["bug"].uri() );
        bugs << bug;
    }
    setBugs( bugs );
}


Qt::ItemFlags Lino::RecommendedBugModel::flags( const QModelIndex& index ) const
{
    if ( index.isValid() &&
         index.row() == 0 &&
         BugModel::rowCount( QModelIndex() ) == 0 ) {
        return Qt::ItemIsEnabled;
    }
    return BugModel::flags( index );
}


QVariant Lino::RecommendedBugModel::data( const QModelIndex& index, int role ) const
{
    if ( index.isValid() &&
         index.row() == 0 &&
         BugModel::rowCount( QModelIndex() ) == 0 ) {
        if ( role == Qt::DisplayRole ) {
            if ( m_parentBug.isValid() )
                return m_requestFinished ? i18n( "No related bugs found" ) : i18n( "Loading..." );
            else
                return i18n( "No bug selected" );
        }
    }
    return BugModel::data( index, role );
}


int Lino::RecommendedBugModel::rowCount( const QModelIndex& parent ) const
{
    int rc = BugModel::rowCount( parent );
    if ( !parent.isValid() && rc == 0 ) {
        return 1;
    }
    return rc;
}

#include "recommendedbugmodel.moc"
