/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "buggertray.h"
#include "buggerpopup.h"
#include "lino.h"
#include "bom.h"
#include "swimconnection.h"
#include "webpage.h"

#include <Soprano/Client/SparqlModel>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>
#include <Soprano/LiteralValue>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/RDFS>

#include <Solid/DeviceNotifier>
#include <Solid/Device>

#include <KDebug>


BuggerTray::BuggerTray()
    : KSystemTrayIcon( "nepomuk" ),
      m_lastRequest( 0 )
{
    connect( Solid::DeviceNotifier::instance(), SIGNAL( deviceAdded( QString ) ),
             this, SLOT( slotDeviceAdded( QString ) ) );
    show();
}


namespace {
    /**
     * Very basic query string conversion. Simply search for all words
     * in the string.
     */
    QString fullTextQueryString( const QString& s ) {
        QString qs( s );
        qs.replace( QRegExp( "\\W" ), QString( ' ' ) );
        qs = qs.simplified();
        qs.replace( ' ', QLatin1String( " AND " ) );
        return qs;
    }
}

void BuggerTray::slotDeviceAdded( const QString& udi )
{
    Solid::Device dev( udi );

    //
    // let's try to find some related things
    // since we do not have all hardware details locally we simply select the
    // related stuff as optional. Thus, we get possible hardware components
    // and related things at the same time
    //
    kDebug() << "Found device:" << dev.product() << dev.vendor();
    if ( !dev.product().isEmpty() && !dev.vendor().isEmpty() ) {
        QString query = QString( "select ?uri ?type ?label ?hw where { "
                                 "?hw a <%1> . "
                                 "OPTIONAL { ?uri ?relation ?hw . "
                                 "?uri <%2> ?label . "
                                 "?uri a ?type . } . "
                                 "{ ?hw <%2> ?compLabel . "
                                 "?compLabel bif:contains '%3' . } "
                                 "UNION "
                                 "{ ?hw <%4> ?manuf . "
                                 "?manuf <%2> ?manufLabel . "
                                 "?manufLabel bif:contains '%5' . } "
                                 "}" )
                        .arg( Lino::Vocabulary::LINO::Hardware().toString() )
                        .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                        .arg( fullTextQueryString( dev.product() ) )
                        .arg( Lino::Vocabulary::LINO::hardwareManufacturer().toString() )
                        .arg( fullTextQueryString( dev.vendor() ) );
        kDebug() << "device query:" << query;
        if ( m_lastRequest )
            m_lastRequest->disconnect( this );
        m_lastRequest = SwimConnection::instance()->executeQueryAsync( query, Soprano::Query::QueryLanguageSparql );
        m_device = dev;
        connect( m_lastRequest, SIGNAL( resultReady( Soprano::Util::AsyncResult* ) ),
                 this, SLOT( slotResultReady( Soprano::Util::AsyncResult* ) ) );
    }
}

static const int s_maxResults = 10;

// FIXME: only use the others list and handle the rest in the popup in a generic manner
void BuggerTray::slotResultReady( Soprano::Util::AsyncResult* result )
{
    kDebug();

    m_lastRequest = 0;

    //
    // Check the results for relations and possible hardware components
    //
    Soprano::QueryResultIterator it = result->queryResultIterator();
    WebPage::List bugs;
    WebPage::List threads;
    WebPage::List others;
    QList<QUrl> possibleHwMatches;
    int cnt = 0;
    while ( it.next() ) {
        QString label = it["label"].toString();
        QUrl uri = it["uri"].uri();
        QUrl type = it["type"].uri();
        QUrl hw = it["hw"].uri();

        if ( uri.isValid() ) {
            WebPage page( uri, label, type );

            //
            // We can get the same result multiple times with different types!
            //
            if ( !bugs.contains( page ) && !threads.contains( page ) && !others.contains( page ) && ++cnt < s_maxResults ) {
                if ( type == Lino::Vocabulary::LINO::Bug() )
                    bugs << page;
                else if ( type == Lino::Vocabulary::LINO::ForumThread() )
                    threads << page;
                else
                    others << page;
            }
        }

        possibleHwMatches << hw;
    }

    //
    // Now see if we have anything locally that links to the possible hardware components
    //
    if ( !possibleHwMatches.isEmpty() && cnt < s_maxResults ) {
        QStringList subQueries;
        foreach( const QUrl& uri, possibleHwMatches ) {
            subQueries << QString( "?uri ?relation <%1> . " ).arg( uri.toString() );
        }
        QString localQuery = QString( "select ?uri ?type ?label where { "
                                      "?uri a ?type . "
                                      "?uri <%1> ?label . "
                                      "{ %2 } . "
                                      "FILTER(?type != <%3>) . }" ) // a small hack
                             .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                             .arg( subQueries.join( " } UNION { " ) )
                             .arg( Soprano::Vocabulary::RDFS::Resource().toString() );
        kDebug() << "Local query:" << localQuery;
        it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( localQuery, Soprano::Query::QueryLanguageSparql );
        while ( it.next() ) {
            QString label = it["label"].toString();
            QUrl uri = it["uri"].uri();
            QUrl type = it["type"].uri();

            WebPage page( uri, label, type );

            //
            // We can get the same result multiple times with different types!
            //
            if ( !bugs.contains( page ) && !threads.contains( page ) && !others.contains( page ) && ++cnt < s_maxResults ) {
                if ( type == Lino::Vocabulary::LINO::Bug() )
                    bugs << page;
                else if ( type == Lino::Vocabulary::LINO::ForumThread() )
                    threads << page;
                else
                    others << page;
            }
        }
    }

    if ( bugs.count() || threads.count() || others.count() ) {
        kDebug() << "Found" << bugs.count() << "bugs and" << threads.count() << "threads and" << others.count() << "other";
        BuggerPopup::showBugsForDevice( m_device, bugs, threads, others, this );
    }
    else
        kDebug() << "No bugs found";
}

#include "buggertray.moc"
