/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "annotationuploader.h"
#include "annotationuploader_p.h"
#include "lino.h"
#include "bom.h"
#include "bug.h"
#include "settings.h"
#include "swimconnection.h"

#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/Client/SparqlModel>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Util/AsyncResult>

#include <nepomuk/resourcemanager.h>
#include <nepomuk/tag.h>
#include <nepomuk/variant.h>
#include <nepomuk/property.h>

#include <KDebug>
#include <KLocale>

using namespace Soprano;


void Lino::AnnotationUploader::Private::run()
{
    foreach( Nepomuk::Resource localResource, m_resources ) {
        Nepomuk::Resource localResource = m_resources.takeFirst();

        emit infoMessage( q, i18n("Uploading annotations for '%1'", localResource.genericLabel() ) );

        // get the remote resource (the lino:Bug)
        Nepomuk::Resource remoteResource( localResource.resourceUri(), localResource.resourceType(), SwimConnection::instance()->resourceManager() );
        remoteResource.setTypes( localResource.types() );

        // copy all the tags from the local one to the remote one
        foreach( const Nepomuk::Tag& tag, localResource.tags() ) {
            remoteResource.addTag( Nepomuk::Tag( tag.genericLabel(), SwimConnection::instance()->resourceManager() ) );
        }

        //
        // We upload all annotations except those to local files
        // during that process we check if all related resources exist on the server and if not,
        // create a dummy resource consisting of a type and a label
        //
        QHash<QUrl, Nepomuk::Variant> props = localResource.properties();
        for ( QHash<QUrl, Nepomuk::Variant>::const_iterator it = props.constBegin();
              it != props.constEnd(); ++it ) {

            Nepomuk::Types::Property prop = it.key();
            const Nepomuk::Variant& val = it.value();

            if ( val.simpleType() == qMetaTypeId<Nepomuk::Resource>() ) {

                // special case: tags, handled above
                // ignore properties that are created by the framework
                if ( prop.uri() == Soprano::Vocabulary::NAO::hasTag() ||
                     prop.uri() == Soprano::Vocabulary::NAO::lastModified() ||
                     prop.uri() == Soprano::Vocabulary::NAO::created() )
                    continue;

                QList<Nepomuk::Resource> relatedResources = val.toResourceList();
                for ( int i = 0; i < relatedResources.count(); ++i ) {
                    Nepomuk::Resource& relatedResource = relatedResources[i];

                    // skip local files
                    if ( KUrl( relatedResource.resourceUri() ).isLocalFile() ) {
                        continue;
                    }

                    // FIXME: this should be handled within libnepomuk: it should be possible
                    //        to simply do: remoteResource.addProperty( relatedResource );
                    //        and ResourceData should copy the resource to the other manager
                    Nepomuk::Resource remoteRelatedRes( relatedResource.resourceUri(),
                                                        relatedResource.resourceType(),
                                                        SwimConnection::instance()->resourceManager() );
                    if ( !remoteRelatedRes.exists() ) {
                        remoteRelatedRes.setTypes( relatedResource.types() );
                        remoteRelatedRes.setLabel( relatedResource.label() );
                    }

                    remoteResource.addProperty( prop.uri(), remoteRelatedRes );
                }
            }
            else {
                // literal values can be stored without problems
                if ( prop.maxCardinality() == 1 ) {
                    remoteResource.setProperty( prop.uri(), val );
                }
                else {
                    remoteResource.addProperty( prop.uri(), val );
                }
            }
        }
    }
}


void Lino::AnnotationUploader::Private::_k_slotFinished()
{
    q->emitResult();
}



Lino::AnnotationUploader::AnnotationUploader( QObject* parent )
    : KJob( parent ),
      d(new Private())
{
    d->q = this;
    connect( d, SIGNAL( infoMessage( KJob*, QString, QString ) ),
             this, SIGNAL( infoMessage( KJob*, QString, QString ) ),
             Qt::QueuedConnection );
    connect( d, SIGNAL( finished() ),
             this, SLOT( _k_slotFinished() ),
             Qt::QueuedConnection );
}


Lino::AnnotationUploader::~AnnotationUploader()
{
    delete d;
}


void Lino::AnnotationUploader::start()
{
    emit description( this, i18n("Uploading annotations") );
    d->start();
}


Lino::AnnotationUploader* Lino::AnnotationUploader::uploadAnnotations( const Nepomuk::Resource& res )
{
    return uploadAnnotations( QList<Nepomuk::Resource>() << res );
}


Lino::AnnotationUploader* Lino::AnnotationUploader::uploadAnnotations( const QList<Nepomuk::Resource>& res )
{
    AnnotationUploader* job = new AnnotationUploader();
    job->d->m_resources = res;
    job->start();
    return job;
}

#include "annotationuploader.moc"
#include "annotationuploader_p.moc"
