/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "annotationrecommendationmodel.h"
#include "swimconnection.h"
#include "lino.h"
#include "wf.h"
#include "utils.h"

#include <nepomuk/simpleannotation.h>
#include <nepomuk/thing.h>
#include <nepomuk/variant.h>
#include <nepomuk/property.h>

#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>

#include <KLocale>

#include <QtCore/QMimeData>


namespace {
    class ForumThreadAnnotation : public Nepomuk::SimpleAnnotation
    {
    public:
        ForumThreadAnnotation( const Nepomuk::Resource& res,
                               const QUrl& forumPost,
                               const QString& label,
                               const QUrl& relation,
                               const QString& creator,
                               QObject* parent );

    protected:
        void doCreate();

    private:
        Nepomuk::Resource m_forumPost;
        QString m_label;
    };


    ForumThreadAnnotation::ForumThreadAnnotation( const Nepomuk::Resource& res,
                                                  const QUrl& forumPost,
                                                  const QString& label,
                                                  const QUrl& relation,
                                                  const QString& creator,
                                                  QObject* parent )
        : SimpleAnnotation( res, parent ),
          m_forumPost( forumPost ),
          m_label( label )
    {
        setValue( m_forumPost );
        setProperty( relation );
        setComment( i18n( "%1 created relation to: '%1'", creator, label ) );
    }


    void ForumThreadAnnotation::doCreate()
    {
        SimpleAnnotation::doCreate();
        // make sure we have the correct types
        m_forumPost.addType( Lino::Vocabulary::LINO::ForumThread() );
        m_forumPost.setLabel( m_label );
    }
}


AnnotationRecommendationModel::AnnotationRecommendationModel( QObject* parent )
    : AbstractAnnotationModel( parent ),
      m_lastResult( 0 )
{
}


AnnotationRecommendationModel::~AnnotationRecommendationModel()
{
}


static const char* s_fakeMimeType = "internalfakestuff/x-row-number";

QMimeData* AnnotationRecommendationModel::mimeData( const QModelIndexList& indexes ) const
{
    QMimeData* mime = new QMimeData();
    QByteArray data;
    QDataStream s( &data, QIODevice::WriteOnly );
    foreach( const QModelIndex& index, indexes ) {
        s << index.row();
    }
    mime->setData( s_fakeMimeType, data );
    return mime;
}


QStringList AnnotationRecommendationModel::mimeTypes( ) const
{
    return QStringList() << s_fakeMimeType;
}


QModelIndexList AnnotationRecommendationModel::indexListFromMimeData( const QMimeData* mimeData ) const
{
    QByteArray data = mimeData->data( s_fakeMimeType );
    QDataStream s( data );
    QModelIndexList indices;
    while ( !s.atEnd() ) {
        int row;
        s >> row;
        indices << index( row, 0 );
    }
    return indices;
}


Qt::ItemFlags AnnotationRecommendationModel::flags( const QModelIndex& index ) const
{
    Qt::ItemFlags flags = AbstractAnnotationModel::flags( index );
    if ( index.isValid() )
        flags |= Qt::ItemIsDragEnabled;
    return flags;
}


namespace {
    Nepomuk::SimpleAnnotation* createMandivaAnnotation( const Nepomuk::Resource& res, const QUrl& uri, const QString& label )
    {
        Nepomuk::SimpleAnnotation* anno = new Nepomuk::SimpleAnnotation( res,
                                                                         Lino::Vocabulary::LINO::isRelatedToOS(),
                                                                         Nepomuk::Variant( Nepomuk::Resource( uri ) ) );
        anno->setLabel( label );
        anno->setComment( i18n( "Is related to linux distribution '%1'", label ) );
        return anno;
    }
}


void AnnotationRecommendationModel::createMandrivaAnnotations()
{
    // recommend different mandriva versions by default (The query is not great.)
    if ( resource().resourceUri().isEmpty() ||
         !Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString( "ask where { "
                                                                                    "<%1> <%2> ?distrib . "
                                                                                    "?distrib a ?type . "
                                                                                    "?type <%3> <%4> . }" )
                                                                           .arg( QString::fromAscii( resource().resourceUri().toEncoded() ) )
                                                                           .arg( Lino::Vocabulary::LINO::isRelatedToOS().toString() )
                                                                           .arg( Soprano::Vocabulary::RDFS::subClassOf().toString() )
                                                                           .arg( Lino::Vocabulary::LINO::Linux().toString() ),
                                                                           Soprano::Query::QueryLanguageSparql ).boolValue() ) {
        addNewAnnotation( createMandivaAnnotation( resource(), Lino::Vocabulary::LINO::Mandriva2008(), "Mandriva 2008" ) );
        addNewAnnotation( createMandivaAnnotation( resource(), Lino::Vocabulary::LINO::Mandriva2008_1(), "Mandriva 2008 Spring" ) );
        addNewAnnotation( createMandivaAnnotation( resource(), Lino::Vocabulary::LINO::Mandriva2009(), "Mandriva 2009" ) );
        addNewAnnotation( createMandivaAnnotation( resource(), Lino::Vocabulary::LINO::Mandriva2009_1(), "Mandriva 2009 Spring" ) );
    }
}


void AnnotationRecommendationModel::createStateAnnotations()
{
    // propose to close the bug or forum post
    QUrl type = Lino::determineResourceTypeFromUrl( resource().resourceUri() );
    if ( type == Lino::Vocabulary::LINO::Bug() || type == Lino::Vocabulary::LINO::ForumThread() ) {
        if ( !Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( QString( "ask where { "
                                                                                        "<%1> <%2> ?state . "
                                                                                        "FILTER(?state = <%3> || ?state = <%4> || ?state = <%5>) . }" )
                                                                               .arg( QString::fromAscii( resource().resourceUri().toEncoded() ) )
                                                                               .arg( Lino::Vocabulary::WF::state().toString() )
                                                                               .arg( Lino::Vocabulary::LINO::Resolved().toString() )
                                                                               .arg( Lino::Vocabulary::LINO::Verified().toString() )
                                                                               .arg( Lino::Vocabulary::LINO::Closed().toString() ),
                                                                               Soprano::Query::QueryLanguageSparql ).boolValue() ) {
            Nepomuk::SimpleAnnotation* anno = new Nepomuk::SimpleAnnotation( resource(),
                                                                             Lino::Vocabulary::WF::state(),
                                                                             Nepomuk::Resource( Lino::Vocabulary::LINO::Resolved() ) );
            anno->setLabel( "resolved" );
            anno->setComment( i18n( "Mark as 'resolved'. (for further states use annotate tool)" ) );
            addNewAnnotation( anno );
        }
    }
}


void AnnotationRecommendationModel::doUpdateAnnotations()
{
    // - find remote stuff with the same tags
    // - find local stuff with the same tags
    // - find remote stuff that someone else related to this but not we

    if ( m_lastResult )
        m_lastResult->disconnect( this );

    createMandrivaAnnotations();
    createStateAnnotations();

//     m_lastResult = SwimConnection::instance()->executeQueryAsync( QString( "select ?post where { "
//                                                                            "?thing <%1> ?post . "
//                                                                            "?thing a <%2> . "
//                                                                            "?thing <%3> ?tag . "
//                                                                            "{ ?tag <%4> xxx .} UNION { ?tag <%4> yyy . } ..." )
//                                                                   .arg( Nepomuk::Vocabulary::PIMO::groundingOccurrence().toString() )
//                                                                   .arg( Lino::Vocabulary::LINO::ForumThread().toString() )
//                                                                   .arg( Soprano::Vocabulary::NAO::hasTag().toString() )
//                                                                   .arg( tagName ).arg( tagName2 ).... );
    // query related forum posts
    m_lastResult = SwimConnection::instance()->executeQueryAsync( QString( "select distinct ?post ?label ?creator ?relation where { "
                                                                           "?post a ?type . "
                                                                           "?post <%2> ?label . "
                                                                           "graph ?g { <%3> ?relation ?post . } . "
                                                                           "?g <%4> ?creator . "
                                                                           "FILTER(?creator != <%5>) . "
                                                                           "FILTER(?type = <%1> || ?type = <%6> || ?type = <%7> || ?type = <%8> ) . }" )
                                                                  .arg( Lino::Vocabulary::LINO::ForumThread().toString() )
                                                                  .arg( Soprano::Vocabulary::NAO::prefLabel().toString() )
                                                                  .arg( resource().resourceUri().toString() )
                                                                  .arg( Soprano::Vocabulary::NAO::creator().toString() )
                                                                  .arg( SwimConnection::instance()->userResourceUri().toString() )
                                                                  .arg( Lino::Vocabulary::LINO::Bug().toString() )
                                                                  .arg( Lino::Vocabulary::LINO::Hardware().toString() )
                                                                  .arg( Lino::Vocabulary::LINO::Software().toString() ));
    connect( m_lastResult, SIGNAL( resultReady( Soprano::Util::AsyncResult* ) ),
             this, SLOT( slotRemoteResultReady( Soprano::Util::AsyncResult* ) ) );
}


void AnnotationRecommendationModel::slotRemoteResultReady( Soprano::Util::AsyncResult* result )
{
    if ( !result->lastError() ) {
        // FIXME: ignore those that are already locally done
        Soprano::QueryResultIterator it = result->queryResultIterator();
        while ( it.next() ) {
            QUrl post = it.binding( "post" ).uri();
            QUrl relation = it["relation"].uri();
            QString label = it["label"].toString();
            QString creator = it["creator"].toString().section( '/', -1 );
            addNewAnnotation( new ForumThreadAnnotation( resource(), post, label, relation, creator, this ) );
        }
    }
    m_lastResult = 0;
}

#include "annotationrecommendationmodel.moc"
