/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "clipboardmodel.h"
#include "utils.h"

#include <nepomuk/resource.h>
#include <nepomuk/property.h>

#include <QtCore/QMimeData>


ClipboardModel::ClipboardModel( QObject* parent )
    : QAbstractListModel( parent )
{
}


ClipboardModel::~ClipboardModel()
{
}


Qt::ItemFlags ClipboardModel::flags( const QModelIndex& index ) const
{
    Qt::ItemFlags f = QAbstractListModel::flags( index );
    if ( index.isValid() )
        f |= Qt::ItemIsDragEnabled;
    return f;
}


int ClipboardModel::rowCount( const QModelIndex& parent ) const
{
    return parent.isValid() ? 0 : m_entries.count();
}


QVariant ClipboardModel::data( const QModelIndex& index, int role ) const
{
    if ( index.row() >= 0 && index.row() < m_entries.count() ) {
        const WebPage& entry = m_entries[index.row()];
        if ( role == Qt::DisplayRole ) {
            return QString( "%1 (%2)" ).arg( entry.title(), Nepomuk::Types::Property( Lino::determineResourceTypeFromUrl( entry.url() ) ).label() );
        }
        else if ( role == Qt::ToolTipRole ) {
            return entry.url().url();
        }
        else if ( role == UrlRole ) {
            return entry.url();
        }
        else if ( role == WebPageRole ) {
            return QVariant::fromValue( entry );
        }
    }

    return QVariant();
}


QStringList ClipboardModel::mimeTypes( ) const
{
    return WebPage::List::mimeDataTypes();
}


QMimeData* ClipboardModel::mimeData( const QModelIndexList& indexes ) const
{
    KUrl::List urls;
    WebPage::List pages;
    foreach( const QModelIndex& index, indexes ) {
        urls << m_entries[index.row()].url();
        pages << m_entries[index.row()];
    }
    QMimeData* data = new QMimeData();
    urls.populateMimeData( data );
    pages.populateMimeData( data );
    return data;
}


void ClipboardModel::addUrl( const KUrl& url, const QString& title )
{
    addPage( WebPage( url, title ) );
}


void ClipboardModel::addPage( const WebPage& entry )
{
    if ( m_entries.contains( entry ) ) {
        // get it to the front
        m_entries.removeAll( entry );
    }
    m_entries.insert( 0, entry );
    reset();
}


void ClipboardModel::clear()
{
    m_entries.clear();
    reset();
}

#include "clipboardmodel.moc"
