/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "sidebarwidget.h"
#include "resourcetaggingwidget.h"
#include "clipboardmodel.h"
#include "lino.h"
#include "annotationrecommendationmodel.h"
#include "utils.h"
#include "webpage.h"
#include "annotationuploader.h"
#include "settings.h"
#include "swimconnection.h"

#include <nepomuk/thing.h>
#include <nepomuk/variant.h>
#include <nepomuk/annotation.h>

#include <nepomuk/pimo.h>
#include <nepomuk/annotationdisplay.h>

#include <Soprano/QueryResultIterator>

#include <QtGui/QStandardItemModel>
#include <QtGui/QDragEnterEvent>
#include <QtGui/QProgressBar>

#include <KIcon>
#include <KUrl>
#include <KDebug>
#include <KMessageBox>
#include <kconfigdialog.h>
#include <KProgressDialog>

#include <khtmlview.h>
#include <khtml_part.h>
#include <dom/html_document.h>


namespace {
    QUrl relationsForType( const QUrl& type ) {
        if ( type == Lino::Vocabulary::LINO::Hardware() )
            return Lino::Vocabulary::LINO::hardware();
        else if ( type == Lino::Vocabulary::LINO::Bug() )
            return Lino::Vocabulary::LINO::relatedBug();
        else
            return Nepomuk::Vocabulary::PIMO::isRelated();
    }
}


SidebarWidget::SidebarWidget( QWidget* parent )
    : QWidget( parent ),
      m_uploadProgressDialog( 0 )
{
    setupUi( this );

    m_taggingWidget->setEnabled(false);

    m_clipboard = new ClipboardModel( m_viewClipboard );
    m_viewClipboard->setModel( m_clipboard );
    m_viewClipboard->setDragDropMode( QAbstractItemView::DragOnly );

    m_clearClipboardButton->setIcon( KIcon( "edit-clear-list" ) );
    m_addToClipboardButton->setIcon( KIcon( "list-add" ) );
    m_uploadAnnotationsButton->setIcon( KIcon( "arrow-up" ) );
    m_configButton->setIcon( KIcon( "configure" ) );

    m_annotationDisplay->installEventFilter( this );
    m_annotationDisplay->setAcceptDrops( true );
    m_annotationDisplay->setPropertyRemovalEnabled( true );
    m_annotationDisplay->addHiddenProperty( Nepomuk::Vocabulary::PIMO::groundingOccurrence() );

    m_recommendationModel = new AnnotationRecommendationModel( m_recommendationView );
    m_recommendationView->setModel( m_recommendationModel );
    m_recommendationView->setDragDropMode( QAbstractItemView::DragOnly );

    connect( m_clearClipboardButton, SIGNAL( clicked() ),
             m_clipboard, SLOT(clear() ) );
    connect( m_addToClipboardButton, SIGNAL( clicked() ),
             this, SLOT( slotAddUrl() ) );
    connect( m_uploadAnnotationsButton, SIGNAL( clicked() ),
             this, SLOT( slotUploadAnnotations() ) );
    connect( m_configButton, SIGNAL( clicked() ),
             this, SLOT( slotPreferences() ) );

    // It would be so great to have ResourceManager::resourceChanged working!
    connect( m_annotationWidget, SIGNAL( annotationAdded( Nepomuk::Resource ) ),
             this, SLOT( slotResourceChanged() ) );
    connect( m_taggingWidget, SIGNAL( tagsChanged( QList<Nepomuk::Tag> ) ),
             this, SLOT( slotResourceChanged() ) );
    connect( m_viewClipboard, SIGNAL( doubleClicked( QModelIndex ) ),
             this, SLOT( slotClipboardDoubleClicked( QModelIndex ) ) );
    connect( m_recommendationModel, SIGNAL( annotationCreated( Nepomuk::Annotation* ) ),
             this, SLOT( slotResourceChanged() ) );
}


SidebarWidget::~SidebarWidget()
{
}


void SidebarWidget::setUrl( const KUrl& url )
{
    if ( !checkSettings() )
        return;

    m_url = url;
    m_resource = url;
    m_annotationWidget->setResource( m_resource );
    m_taggingWidget->setResource( m_resource );
    m_taggingWidget->setEnabled( m_url.isValid() );
    m_annotationWidget->setEnabled( m_url.isValid() );
    m_annotationDisplay->setResource( m_resource );
    m_recommendationModel->setResource( m_resource );

    kDebug() << findCurrentDocument().title().string();

    // TODO: read annotations (in this case relations to other forum entries and bugs)
}


KUrl SidebarWidget::currentUrl() const
{
    return m_url;
}


void SidebarWidget::slotAddUrl()
{
    m_clipboard->addUrl( m_url, Lino::cleanupPageTitle( findCurrentDocument().title().string() ) );
}


void SidebarWidget::slotUploadAnnotations()
{
    Nepomuk::Resource res( m_url );
    if ( res.exists() ) {
        if ( !m_uploadProgressDialog ) {
            m_uploadProgressDialog = new KProgressDialog( this, i18n( "Uploading Annotations" ) );
            m_uploadProgressDialog->setButtonText( i18n( "Background" ) );
            m_uploadProgressDialog->progressBar()->setMaximum( 0 );
        }
        m_uploadProgressDialog->setLabelText( i18n( "Uploading Anotations for '%1'. This can take a while...", res.genericLabel() ) );

        connect( Lino::AnnotationUploader::uploadAnnotations( res ), SIGNAL( result( KJob* ) ),
                 this, SLOT( slotAnnotationsUploaded( KJob* ) ) );

        m_uploadAnnotationsButton->setEnabled( false );
        m_uploadProgressDialog->exec();
    }
    else {
        KMessageBox::sorry( this, i18n( "Nothing to upload" ) );
    }
}


void SidebarWidget::slotAnnotationsUploaded( KJob* job )
{
    m_uploadProgressDialog->close();
    m_uploadAnnotationsButton->setEnabled( true );

    if ( job->error() ) {
        KMessageBox::error( this, i18n( "Annotations upload failed (%1)", job->errorString() ) );
    }
    else {
        KMessageBox::information( this, i18n( "Annotations uploaded successfully" ) );
    }
}


bool SidebarWidget::eventFilter( QObject* obj, QEvent* event )
{
    if ( obj == m_annotationDisplay ) {
        if ( event->type() == QEvent::DragEnter ) {
            QDragEnterEvent* de = static_cast<QDragEnterEvent*>( event );
            event->setAccepted( WebPage::List::canDecode( de->mimeData() ) || de->mimeData()->hasFormat( m_recommendationModel->mimeTypes()[0] ) );
            return true;
        }
        else if ( event->type() == QEvent::Drop ) {
            QDropEvent* de = static_cast<QDragEnterEvent*>( event );
            WebPage::List pages = WebPage::List::fromMimeData( de->mimeData() );
            if ( pages.count() ) {
                WebPage page = pages[0];
                Nepomuk::Resource res( page.url() );
                res.setLabel( page.title() );
                QUrl thingType = Lino::determineResourceTypeFromUrl( page.url() );
                res.addType( thingType );
                m_resource.addProperty( relationsForType( thingType ), Nepomuk::Variant( res ) );
                slotResourceChanged();
            }
            else if ( de->mimeData()->hasFormat( m_recommendationModel->mimeTypes()[0] ) ) {
                QModelIndexList indices = m_recommendationModel->indexListFromMimeData( de->mimeData() );
                foreach( const QModelIndex& index, indices ) {
                    m_recommendationModel->annotationForIndex( index )->create();
                }
            }
        }
    }

    return QWidget::eventFilter( obj, event );
}


// we only set label and type once the resource has actually been saved. Otherwise we would end up with one
// resource for each visited webpage in the store
void SidebarWidget::slotResourceChanged()
{
    m_resource.addType( Lino::determineResourceTypeFromUrl( m_url ) );
    m_resource.setLabel( Lino::cleanupPageTitle( findCurrentDocument().title().string() ) );
}


void SidebarWidget::slotClipboardDoubleClicked( const QModelIndex& index )
{
    KUrl url = index.data( ClipboardModel::UrlRole ).value<KUrl>();
    emit requestURL( url );
}


// HACK to get access to the html view and thus, the currently displayed web page
DOM::HTMLDocument SidebarWidget::findCurrentDocument()
{
    if ( KHTMLView* view = parentWidget()->parentWidget()->parentWidget()->parentWidget()->findChild<KHTMLView*>() ) {
        KHTMLPart* part = view->part();
        return part->htmlDocument();
    }
    else {
        return DOM::HTMLDocument();
    }
}


void SidebarWidget::slotPreferences()
{
    if ( KConfigDialog::showDialog( "settings" ) )  {
        return;
    }
    KConfigDialog* dialog = new KConfigDialog( this, "settings", Settings::self() );
    QWidget* generalSettingsDlg = new QWidget;
    ui_prefs_base.setupUi(generalSettingsDlg);
    dialog->addPage(generalSettingsDlg, i18n("General"), "package_setting");
    connect(dialog, SIGNAL(settingsChanged(QString)), SwimConnection::instance(), SLOT(updateConnectionSettings()));
    dialog->show();
}


bool SidebarWidget::checkSettings()
{
    SwimConnection::instance()->executeQuery( QString( "ask where { ?s ?p ?o . }" ) );
    if ( SwimConnection::instance()->lastError() ) {
        KMessageBox::error( this, i18n( "Could not connect to SWIM server. Please configure credentials" ) );
        return false;
    }
    return true;
}

#include "sidebarwidget.moc"
